"""Package with the logger."""

from .logger import logger
from .types import (
    StreamedItem,
    StreamItemContent,
    StreamItemOutput,
    ToolItem,
    ToolItemContent,
)

__all__ = [
    "StreamItemContent",
    "StreamItemOutput",
    "StreamedItem",
    "ToolItem",
    "ToolItemContent",
    "logger",
]



"""Prices for models and providers."""

from llmax.models import Model, Provider

from .exceptions import PriceNotFoundError

PROMPT_PRICES_PER_1M: dict[Model, float | dict[Provider, float]] = {
    "gpt-4.1": 2.0,
    "gpt-4.1-mini": 0.4,
    "gpt-4.1-nano": 0.1,
    "gpt-4o": {"openai": 2.5, "azure": 2.75},
    "gpt-4o-mini": {"openai": 0.15, "azure": 0.165},
    "o3-mini": {"openai": 1.10, "azure": 1.21},
    "o3-mini-high": {"openai": 1.10, "azure": 1.21},
    "ada-v2": 0.1,
    "gpt-3.5": 0.5,
    "gpt-4-turbo": 10.0,
    "claude-3-haiku": 0.25,
    "claude-3.5-sonnet": 3.0,
    "claude-3.7-sonnet": 3.0,
    "claude-4-sonnet": 3.0,
    "claude-4.5-sonnet": 3.0,
    "claude-4.6-sonnet": 3.0,
    "claude-4.5-opus": 5.0,
    "claude-4.6-opus": 5.0,
    "command-r": 0.5,
    "command-r-plus": 3.0,
    "llama-3-70b-instruct": 3.78,  # To be checked
    "llama-4-scout-17b-16e-instruct": 0.275,  # To be checked
    "llama-4-maverick-17b-128e-instruct-fp8": 0.275,  # To be checked
    "mistral-large": 4.0,
    "mistral-large-2411": 2.0,
    "mistral-small": 1.0,
    "text-embedding-3-large": 0.13,
    "text-embedding-3-small": 0.02,
    "bge-multilingual-gemma2": {"scaleway": 0.115},  # €0.10 → $0.115
    "gpt-5": 1.25,
    "gpt-5.1": 1.25,
    "gpt-5-chat": 1.25,
    "gpt-5-mini": 0.25,
    "gpt-5-nano": 0.05,
    "llama-3.1-8b-instruct": {"scaleway": 0.23},  # €0.20 → $0.23
    "qwen3-235b-a22b-instruct-2507": {"scaleway": 0.8625},  # €0.75 → $0.8625
    "gpt-oss-120b": {"scaleway": 0.1725},  # €0.15 → $0.1725
    "gemma-3-27b-it": {"scaleway": 0.2875},  # €0.25 → $0.2875
    "voxtral-small-24b-2507": {"scaleway": 0.1725},  # €0.15 → $0.1725
    "mistral-small-3.2-24b-instruct-2506": {"scaleway": 0.1725},  # €0.15 → $0.1725
    "llama-3.3-70b-instruct": {"scaleway": 1.035},  # €0.90 → $1.035
    "deepseek-r1-distill-llama-70b": {"scaleway": 1.035},  # €0.90 → $1.035
    "grok-4-1-fast": 0.2,
    "gemini-3-pro-preview": 2,
}

CACHED_PROMPT_PRICES_PER_1M: dict[Model, float | dict[Provider, float]] = {
    "gpt-4.1": 0.5,
    "gpt-4.1-mini": 0.1,
    "gpt-4.1-nano": 0.03,
    "gpt-4o": {"openai": 1.25, "azure": 1.375},
    "gpt-4o-mini": {"openai": 0.075, "azure": 0.083},
    "o3-mini": {"openai": 0.55, "azure": 0.605},
    "o3-mini-high": {"openai": 0.55, "azure": 0.605},
    "gpt-5": 0.125,
    "gpt-5.1": 0.125,
    "gpt-5-chat": 0.125,
    "gpt-5-mini": 0.025,
    "gpt-5-nano": 0.005,
    "gpt-oss-120b": 0.85,
}

COMPLETION_PRICES_PER_1M: dict[Model, float | dict[Provider, float]] = {
    "gpt-4.1": 8.0,
    "gpt-4.1-mini": 1.6,
    "gpt-4.1-nano": 0.4,
    "gpt-4o": {"openai": 10.0, "azure": 11.0},
    "gpt-4o-mini": {"openai": 0.60, "azure": 0.66},
    "o3-mini": {"openai": 4.40, "azure": 4.84},
    "o3-mini-high": {"openai": 4.40, "azure": 4.84},
    "gpt-3.5": 1.5,
    "gpt-4-turbo": 30.0,
    "claude-3-haiku": 1.25,
    "claude-3.5-sonnet": 15.0,
    "claude-3.7-sonnet": 15.0,
    "claude-4-sonnet": 15.0,
    "claude-4.5-sonnet": 15.0,
    "claude-4.6-sonnet": 15.0,
    "claude-4.5-opus": 25.0,
    "claude-4.6-opus": 25.0,
    "command-r": 1.5,
    "command-r-plus": 15.0,
    "llama-3-70b-instruct": 11.34,
    "llama-4-scout-17b-16e-instruct": 1.1,  # To be checked
    "llama-4-maverick-17b-128e-instruct-fp8": 1.1,  # To be checked
    "mistral-large": 12.0,
    "mistral-large-2411": 6.0,
    "mistral-small": 3.0,
    "gpt-5": 10.00,
    "gpt-5.1": 10.00,
    "gpt-5-chat": 10.00,
    "gpt-5-mini": 2.00,
    "gpt-5-nano": 0.40,
    "llama-3.1-8b-instruct": {"scaleway": 0.23},  # €0.20 → $0.23
    "qwen3-235b-a22b-instruct-2507": {"scaleway": 2.5875},  # €2.25 → $2.5875
    "gpt-oss-120b": {"scaleway": 0.69},  # €0.60 → $0.69
    "gemma-3-27b-it": {"scaleway": 0.575},  # €0.50 → $0.575
    "voxtral-small-24b-2507": {"scaleway": 0.4025},  # €0.35 → $0.4025
    "mistral-small-3.2-24b-instruct-2506": {"scaleway": 0.4025},  # €0.35 → $0.4025
    "llama-3.3-70b-instruct": {"scaleway": 1.035},  # €0.90 → $1.035
    "deepseek-r1-distill-llama-70b": {"scaleway": 1.035},  # €0.90 → $1.035
    "grok-4-1-fast": 0.5,
    "gemini-3-pro-preview": 12,
}

TRANSCRIPTION_PRICES_PER_1M: dict[Model, float | dict[Provider, float]] = {
    "whisper-1": 0.006,
    "gpt-4o-transcribe": 0.006,
    "whisper-large-v3": {"scaleway": 0.00345},  # €0.003/minute → $0.00345/minute
}

GENERATION_PRICE_BASE: dict[Model, float | dict[Provider, float]] = {
    "dall-e-3": 0.04,
    "gpt-image-1": 0.08,
    "gemini-3-pro-image-preview": 0.08,
}

GENERATION_PRICES_PER_1M: dict[Model, float | dict[Provider, float]] = {
    "tts-1": 0.000015,
}


def _fetch_price(
    prices: dict[Model, float | dict[Provider, float]],
    model: Model,
    provider: Provider,
) -> float:
    """Fetch price for a given model and provider from the provided dictionary."""
    if model not in prices:
        raise PriceNotFoundError(model)

    prices_for_model = prices[model]
    if isinstance(prices_for_model, dict):
        if provider not in prices_for_model:
            raise PriceNotFoundError(model, provider)
        return prices_for_model[provider]

    return prices_for_model


def get_prompt_price(model: Model, provider: Provider) -> float:
    """Get the prompt price for a model and provider."""
    return _fetch_price(PROMPT_PRICES_PER_1M, model, provider)


def get_cached_prompt_price(model: Model, provider: Provider) -> float:
    """Get the cached prompt price for a model and provider."""
    try:
        return _fetch_price(CACHED_PROMPT_PRICES_PER_1M, model, provider)
    except PriceNotFoundError:
        return _fetch_price(PROMPT_PRICES_PER_1M, model, provider)


def get_completion_price(model: Model, provider: Provider) -> float:
    """Get the completion price for a model and provider."""
    return _fetch_price(COMPLETION_PRICES_PER_1M, model, provider)


def get_stt_price(model: Model, provider: Provider) -> float:
    """Get the audio price for a model and provider."""
    return _fetch_price(TRANSCRIPTION_PRICES_PER_1M, model, provider)


def get_tti_price(model: Model, provider: Provider) -> float:
    """Get the generation price for a model and provider."""
    return _fetch_price(GENERATION_PRICE_BASE, model, provider)


def get_tts_price(model: Model, provider: Provider) -> float:
    """Get the generation price for a model and provider."""
    return _fetch_price(GENERATION_PRICES_PER_1M, model, provider)



"""Chat completion message models."""

from __future__ import annotations

from typing import Any, Literal

from pydantic import BaseModel

__all__ = ["ChatCompletionMessage"]


class ChatCompletionAssistantMessage(BaseModel):
    content: str | list[dict[str, Any]]
    """The contents of the assistant message."""

    role: Literal["assistant"] = "assistant"
    """The role of the messages author, in this case `assistant`."""


class ChatCompletionSystemMessage(BaseModel):
    content: list[str] | str
    """The contents of the system message."""

    role: Literal["system"]
    """The role of the messages author, in this case `system`."""


class ChatCompletionUserMessage(BaseModel):
    content: list[str] | str | list[dict[str, Any]]
    """The contents of the user message."""

    role: Literal["user"]
    """The role of the messages author, in this case `user`."""


class ToolCall(BaseModel):
    function: FunctionCall
    id: str
    type: Literal["function"]
    index: int = 0


class FunctionCall(BaseModel):
    arguments: str | None = None
    name: str


class ChatCompletionToolMessage(BaseModel):
    content: str | None = None
    role: Literal["tool"] = "tool"
    tool_call_id: str


ChatCompletionMessage = (
    ChatCompletionSystemMessage
    | ChatCompletionUserMessage
    | ChatCompletionAssistantMessage
    | ChatCompletionToolMessage
)



"""Anthropic clients."""

import json
from collections.abc import Generator
from typing import Any

import boto3
from botocore.config import Config as BotoConfig
from dateutil import parser
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from llmax.external_clients.exceptions import ProviderNotFoundError
from llmax.external_clients.universal_client.chat_completion_message import (
    ChatCompletionAssistantMessage,
    ChatCompletionSystemMessage,
    ChatCompletionUserMessage,
)
from llmax.external_clients.universal_client.client import UniversalClient
from llmax.messages import Messages
from llmax.models import Deployment, Model
from llmax.utils import logger

Client = Any


MAPPING_FINISH_REASON = {
    "end_turn": "stop",
    "max_tokens": "stop",
}


_boto3_clients: dict[tuple[str, str], Any] = {}


def client_creation_anthropic(
    aws_key: str,
    aws_secret_key: str,
    region_name: str,
) -> Any:
    """Create the anthropic client, reusing cached clients per (key, region)."""
    cache_key = (aws_key, region_name)
    if cache_key not in _boto3_clients:
        _boto3_clients[cache_key] = boto3.Session(
            aws_access_key_id=aws_key,
            aws_secret_access_key=aws_secret_key,
            region_name=region_name,
        ).client(
            "bedrock-runtime",
            config=BotoConfig(
                max_pool_connections=100,
                retries={"max_attempts": 3, "mode": "adaptive"},
            ),
        )
    return _boto3_clients[cache_key]


def anthropic_parsing(response: dict[str, Any]) -> ChatCompletion | None:
    """The parsing from anthropic to openAi."""
    try:
        data = json.loads(response["body"].read())
        input_tokens = response["ResponseMetadata"]["HTTPHeaders"][
            "x-amzn-bedrock-input-token-count"
        ]
        completion_tokens = response["ResponseMetadata"]["HTTPHeaders"][
            "x-amzn-bedrock-output-token-count"
        ]
        chat_completion = {
            "id": response["ResponseMetadata"]["RequestId"],
            "created": int(
                parser.parse(
                    response["ResponseMetadata"]["HTTPHeaders"]["date"],
                ).timestamp(),
            ),
            "model": data["model"],
            "usage": {
                "completion_tokens": completion_tokens,
                "prompt_tokens": input_tokens,
                "total_tokens": completion_tokens + input_tokens,
            },
            "choices": [
                {
                    "finish_reason": MAPPING_FINISH_REASON[data["stop_reason"]],
                    "index": i,
                    "message": {"content": e["text"], "role": data["role"]},
                }
                for i, e in enumerate(data["content"])
            ],
            "object": "chat.completion",
        }
        return ChatCompletion.model_validate(chat_completion)
    except Exception:
        return None


def anthropic_parsing_stream(  # noqa: C901, PLR0912, PLR0915
    response: dict[str, Any],
) -> Generator[ChatCompletionChunk, None, None] | None:
    """The parsing from anthropic to openAi in streaming mode."""
    try:
        request_id = response["ResponseMetadata"]["RequestId"]
        created = int(
            parser.parse(
                response["ResponseMetadata"]["HTTPHeaders"]["date"],
            ).timestamp(),
        )
    except Exception:
        return None

    _object = "chat.completion.chunk"
    model = None
    role = None
    prompt_tokens = None
    completion_tokens = None
    stop_reason = None

    # Track active tool calls
    current_tool_calls = {}
    active_tool_id = None

    for event in response["body"]:
        chunk = json.loads(event["chunk"]["bytes"])
        chunk_type = chunk.get("type")

        # Default values for this chunk
        content = None
        tool_calls = None

        if chunk_type == "message_start":
            model = chunk["message"]["model"]
            role = chunk["message"]["role"]
            prompt_tokens = chunk["message"].get("usage", {}).get("input_tokens")
            completion_tokens = chunk["message"].get("usage", {}).get("output_tokens")

        elif chunk_type == "content_block_start":
            block_index = chunk.get("index", 0)
            content_block = chunk.get("content_block", {})
            block_type = content_block.get("type")

            if block_type == "text":
                # Regular text content block
                pass

            elif block_type == "tool_use":
                # Start of a tool call
                tool_id = content_block.get("id")
                tool_name = content_block.get("name")
                active_tool_id = tool_id

                # Initialize the tool call
                current_tool_calls[tool_id] = {
                    "index": block_index,
                    "id": tool_id,
                    "type": "function",
                    "function": {
                        "name": tool_name,
                        "arguments": "",  # Will be filled with JSON
                    },
                }

                # Create initial tool call to send
                tool_calls = [current_tool_calls[tool_id]]

        elif chunk_type == "content_block_delta":
            delta = chunk.get("delta", {})
            delta_type = delta.get("type")

            if delta_type == "text_delta":
                # Regular text content
                content = delta.get("text", "")

            elif delta_type == "input_json_delta" and active_tool_id:
                # Tool JSON input update
                json_fragment = delta.get("partial_json", "")

                # Update the arguments in the current tool call
                if active_tool_id in current_tool_calls:
                    current_tool_id = active_tool_id
                    current_tool_calls[current_tool_id]["function"]["arguments"] = (
                        json_fragment
                    )

                    # Create tool call update to send
                    tool_calls = [
                        {
                            "index": current_tool_calls[current_tool_id]["index"],
                            "id": current_tool_id,
                            "function": {"arguments": json_fragment},
                        },
                    ]

        elif chunk_type == "content_block_stop":
            if active_tool_id:
                active_tool_id = None

        elif chunk_type == "message_delta":
            if "delta" in chunk and "stop_reason" in chunk["delta"]:
                stop_reason = MAPPING_FINISH_REASON.get(
                    chunk["delta"]["stop_reason"],
                    "stop",
                )

        elif chunk_type == "message_stop":
            prompt_tokens = chunk.get("amazon-bedrock-invocationMetrics", {}).get(
                "inputTokenCount",
            )
            completion_tokens = chunk.get("amazon-bedrock-invocationMetrics", {}).get(
                "outputTokenCount",
            )

        # Only yield if we have content or tool calls
        if content is not None or tool_calls is not None:
            try:
                # Prepare delta based on what we have
                delta = {}
                if content is not None:
                    delta["content"] = content
                if role is not None and chunk_type == "message_start":
                    delta["role"] = role
                if tool_calls is not None:
                    delta["tool_calls"] = tool_calls

                chat_completion_chunk = {
                    "id": request_id,
                    "created": created,
                    "model": model,
                    "usage": {
                        "completion_tokens": completion_tokens,
                        "prompt_tokens": prompt_tokens,
                        "total_tokens": completion_tokens + prompt_tokens
                        if completion_tokens and prompt_tokens
                        else None,
                    },
                    "choices": [
                        {
                            "delta": delta,
                            "index": 0,
                            "finish_reason": stop_reason,
                        },
                    ],
                    "object": _object,
                }
                yield ChatCompletionChunk.model_validate(chat_completion_chunk)
            except Exception as e:
                logger.error(f"Error processing chunk: {e}")
                return None
    return None


def completion_call_anthropic(  # noqa: C901, PLR0912
    client: Any,
    messages: Messages,
    model: Model,
    stream: bool = False,
    *args: Any,  # noqa: ARG001
    **kwargs: Any,
) -> ChatCompletion | None | Generator[ChatCompletionChunk, None, None]:
    """Anthropic call to make a completion."""
    system_message: list[str] = []
    to_remove: list[int] = []

    try:
        counter = 0
        for i, message in enumerate(messages):
            try:
                ChatCompletionAssistantMessage.model_validate(message)
                counter += 1
            except Exception:  # noqa: S110
                pass
            try:
                ChatCompletionSystemMessage.model_validate(message)
                system_message.append(str(message["content"]))
                to_remove.append(i)
                counter += 1
            except Exception:  # noqa: S110
                pass
            try:
                ChatCompletionUserMessage.model_validate(message)
                counter += 1
            except Exception:  # noqa: S110
                pass
        if counter < len(messages):
            logger.error("Incorrect message formatting")
    except Exception:
        return None

    body = {
        "anthropic_version": "bedrock-2023-05-31",
        "max_tokens": 10_000,
        "messages": [
            message for i, message in enumerate(messages) if i not in to_remove
        ],
    }
    if len(system_message) > 0:
        body.update({"system": " ".join(system_message)})

    if "tools" in kwargs:
        tools = kwargs["tools"]
        if tools:
            # Convert OpenAI-style tools to Anthropic format
            anthropic_tools = []
            for tool in tools:
                if tool["type"] == "function":
                    anthropic_tool = {
                        "name": tool["function"]["name"],
                        "description": tool["function"].get("description", ""),
                        "input_schema": tool["function"].get("parameters", {}),
                    }
                    anthropic_tools.append(anthropic_tool)

            if anthropic_tools:
                body.update({"tools": anthropic_tools})

    if "tool_choice" in kwargs:
        tool_choice = kwargs["tool_choice"]
        if isinstance(tool_choice, dict) and tool_choice.get("type") == "function":
            body.update({"tool_choice": tool_choice["function"]["name"]})
        elif tool_choice == "auto":
            body.update({"tool_choice": "auto"})

    if stream:
        response_stream = client.invoke_model_with_response_stream(
            modelId=model,
            body=json.dumps(body),
        )
        return anthropic_parsing_stream(response_stream)
    response = client.invoke_model(modelId=model, body=json.dumps(body))
    return anthropic_parsing(response)


def get_client(deployment: Deployment) -> Client:
    """Get a client for the given deployment."""
    match deployment.provider:
        case "aws-bedrock":
            return UniversalClient(
                client_creation=client_creation_anthropic,
                completion_call=completion_call_anthropic,
                deployment=deployment,
                aws_key=deployment.project_id,
                aws_secret_key=deployment.api_key,
                region_name=deployment.region,
            )
        case _:
            raise ProviderNotFoundError(deployment)



"""This module provides functions to get a client for a given deployment."""

from typing import Any

import httpx

from llmax.external_clients.exceptions import ClientNotFoundError
from llmax.models import Deployment
from llmax.models.models import (
    ANTHROPIC_MODELS,
    COHERE_MODELS,
    GEMINI_MODELS,
    GROK_MODELS,
    META_MODELS,
    MISTRAL_MODELS,
    OPENAI_MODELS,
    SCALEWAY_MODELS,
)

from . import anthropic, cohere, gemini, grok, meta, mistral, openai, scaleway

Client = Any


def get_client(  # noqa: PLR0911
    deployment: Deployment,
    http_client: httpx.Client | None = None,
) -> Client:
    """Get a client for the given deployment.

    Args:
        deployment: The deployment configuration
        http_client: Optional httpx client for OpenAI/Azure providers

    Returns:
        The client for the deployment
    """
    deployment.validate()
    match deployment.model:
        case model if model in OPENAI_MODELS:
            return openai.get_client(deployment, http_client=http_client)
        case model if model in MISTRAL_MODELS:
            return mistral.get_client(deployment, http_client=http_client)
        case model if model in COHERE_MODELS:
            return cohere.get_client(deployment, http_client=http_client)
        case model if model in META_MODELS:
            return meta.get_client(deployment, http_client=http_client)
        case model if model in ANTHROPIC_MODELS:
            return anthropic.get_client(deployment)
        case model if model in GEMINI_MODELS:
            return gemini.get_client(deployment, http_client=http_client)
        case model if model in SCALEWAY_MODELS:
            return scaleway.get_client(deployment, http_client=http_client)
        case model if model in GROK_MODELS:
            return grok.get_client(deployment, http_client=http_client)
        case _:
            raise ClientNotFoundError(deployment)


def get_aclient(  # noqa: PLR0911
    deployment: Deployment,
    http_client: httpx.AsyncClient | None = None,
) -> Client:
    """Get an async client for the given deployment.

    Args:
        deployment: The deployment configuration
        http_client: Optional async httpx client for OpenAI/Azure providers

    Returns:
        The async client for the deployment
    """
    deployment.validate()
    match deployment.model:
        case model if model in OPENAI_MODELS:
            return openai.get_aclient(deployment, http_client=http_client)
        case model if model in MISTRAL_MODELS:
            return mistral.get_aclient(deployment, http_client=http_client)
        case model if model in COHERE_MODELS:
            return cohere.get_aclient(deployment, http_client=http_client)
        case model if model in META_MODELS:
            return meta.get_aclient(deployment, http_client=http_client)
        case model if model in GEMINI_MODELS:
            return gemini.get_aclient(deployment, http_client=http_client)
        case model if model in SCALEWAY_MODELS:
            return scaleway.get_aclient(deployment, http_client=http_client)
        case model if model in GROK_MODELS:
            return grok.get_aclient(deployment, http_client=http_client)
        case _:
            raise ClientNotFoundError(deployment)



"""Fake LLM messages."""

import json
import random
import time
from collections.abc import Generator
from typing import Any

from openai.types.chat import ChatCompletionChunk
from openai.types.chat.chat_completion_chunk import Choice, ChoiceDelta


def to_completion_chunk(message: str, model: str) -> ChatCompletionChunk:
    """Convert a message to a completion chunk."""
    return ChatCompletionChunk(
        id="fake",
        choices=[
            Choice(
                delta=ChoiceDelta(content=message + ""),
                finish_reason="stop",
                index=0,
            ),
        ],
        created=1708363210,
        model=model,
        object="chat.completion.chunk",
    )


def fake_llm(
    message: str,
    stream: bool = True,
    send_empty: bool = False,
) -> Generator[str, None, None]:
    """Generate fake LLM messages."""
    if send_empty:
        yield f"0: {json.dumps('', separators=(',', ':'))}\n\n"
        return

    if stream:
        for word in message.split(" "):
            yield f"0: {json.dumps(word, separators=(',', ':'))}\n\n"
            time.sleep(0.1 * random.random())
    else:
        yield f"0: {json.dumps(message, separators=(',', ':'))}\n\n"
    return


def fake_tool_llm(
    tool_name: str,
    tool_id: str,
    args: dict[str, Any],
) -> str:
    """Generate fake LLM messages compatible with useChat."""
    tool_call = {
        "toolCallId": tool_id,
        "toolName": tool_name,
        "args": args,
    }

    return f"9:{json.dumps(tool_call, separators=(',', ':'))}\n\n"



"""Main file to test image generation and editing features."""

import asyncio
import os
from pathlib import Path

from dotenv import load_dotenv

from llmax.clients import MultiAIClient
from llmax.models import Deployment, Model
from llmax.utils import logger

load_dotenv()

OUTPUT_DIR = Path("image_test_output")

# ---------------------------------------------------------------------------
# Deployments
# ---------------------------------------------------------------------------

def build_client() -> MultiAIClient:
    """Build a MultiAIClient with image-capable deployments.

    Reads from environment variables. For each model, the following variables
    are supported (GPT_IMAGE_1 shown as example):

        LLMAX_GPT_IMAGE_1_API_KEY        required
        LLMAX_GPT_IMAGE_1_PROVIDER       "openai" (default) | "azure"
        LLMAX_GPT_IMAGE_1_ENDPOINT       required when provider is "azure"
        LLMAX_GPT_IMAGE_1_DEPLOYMENT     deployment name (defaults to model name)
        LLMAX_GPT_IMAGE_1_API_VERSION    Azure API version (defaults to 2025-04-01-preview)

        LLMAX_GEMINI_IMAGE_API_KEY       required for Gemini tests
    """
    gpt_provider = os.getenv("LLMAX_GPT_IMAGE_1_PROVIDER", "openai")
    deployments: dict[Model, Deployment] = {
        "gpt-image-1": Deployment(
            model="gpt-image-1",
            provider=gpt_provider,  # type: ignore[arg-type]
            deployment_name=os.getenv("LLMAX_GPT_IMAGE_1_DEPLOYMENT", "gpt-image-1"),
            api_key=os.getenv("LLMAX_GPT_IMAGE_1_API_KEY", ""),
            endpoint=os.getenv("LLMAX_GPT_IMAGE_1_ENDPOINT", ""),
            api_version=os.getenv("LLMAX_GPT_IMAGE_1_API_VERSION", "2025-04-01-preview"),
        ),
        "gemini-3-pro-image-preview": Deployment(
            model="gemini-3-pro-image-preview",
            provider="gemini",
            deployment_name="gemini-3-pro-image-preview",
            api_key=os.getenv("LLMAX_GEMINI_IMAGE_API_KEY", ""),
        ),
    }
    return MultiAIClient(deployments=deployments, fallback_models={})


# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------

def save(name: str, data: bytes) -> Path:
    """Save image bytes to the output directory and return the path."""
    OUTPUT_DIR.mkdir(exist_ok=True)
    path = OUTPUT_DIR / name
    path.write_bytes(data)
    logger.info(f"  → saved {path} ({len(data):,} bytes)")
    return path


# ---------------------------------------------------------------------------
# Tests
# ---------------------------------------------------------------------------

async def test_text_to_image(client: MultiAIClient) -> bytes:
    """Test text_to_image with the latest parameters."""
    logger.info("=== text_to_image ===")

    image_bytes = await client.text_to_image(
        model="gpt-image-1",
        prompt="A serene Japanese garden at sunrise, watercolor style",
        quality="medium",
        aspect_ratio="16:9",
        background="opaque",
        output_format="png",
    )
    save("text_to_image.png", image_bytes)
    return image_bytes


async def test_text_to_image_gemini(client: MultiAIClient) -> None:
    """Test text_to_image with a Gemini model."""
    logger.info("=== text_to_image (Gemini) ===")

    image_bytes = await client.text_to_image(
        model="gemini-3-pro-image-preview",
        prompt="A futuristic city skyline at night, neon lights",
        quality="high",
        aspect_ratio="16:9",
    )
    save("text_to_image_gemini.png", image_bytes)


async def test_stream_text_to_image(client: MultiAIClient) -> None:
    """Test stream_text_to_image, saving each progressive frame."""
    logger.info("=== stream_text_to_image ===")

    frame_index = 0
    async for frame_bytes in client.stream_text_to_image(
        model="gpt-image-1",
        prompt="A majestic mountain range covered in snow, photorealistic",
        quality="medium",
        aspect_ratio="16:9",
        output_format="png",
        partial_images=2,
    ):
        save(f"stream_text_to_image_frame_{frame_index:02d}.png", frame_bytes)
        frame_index += 1

    logger.info(f"  → {frame_index} frame(s) received")


async def test_edit_image(client: MultiAIClient, base_image: bytes) -> bytes:
    """Test edit_image with a single image."""
    logger.info("=== edit_image (single) ===")

    image_input = ("base.png", base_image, "image/png")
    edited_bytes = await client.edit_image(
        model="gpt-image-1",
        prompt="Add a small red torii gate in the foreground",
        image=image_input,
        quality="medium",
        background="opaque",
        output_format="png",
    )
    save("edit_image_single.png", edited_bytes)
    return edited_bytes


async def test_edit_image_multi(client: MultiAIClient, base_image: bytes) -> None:
    """Test edit_image with multiple reference images."""
    logger.info("=== edit_image (multi-image) ===")

    # Generate a second reference image to use alongside the base one
    second_image_bytes = await client.text_to_image(
        model="gpt-image-1",
        prompt="A traditional Japanese paper lantern festival at night, watercolor style",
        quality="low",
        aspect_ratio="1:1",
        output_format="png",
    )
    save("edit_image_multi_ref2.png", second_image_bytes)

    edited_bytes = await client.edit_image(
        model="gpt-image-1",
        prompt="Blend these two scenes into a single cohesive artistic composition",
        image=[
            ("scene1.png", base_image, "image/png"),
            ("scene2.png", second_image_bytes, "image/png"),
        ],
        quality="medium",
        output_format="png",
    )
    save("edit_image_multi.png", edited_bytes)


async def test_edit_image_gemini(client: MultiAIClient, base_image: bytes) -> None:
    """Test edit_image with a Gemini model."""
    logger.info("=== edit_image (Gemini) ===")

    image_input = ("base.png", base_image, "image/png")
    edited_bytes = await client.edit_image(
        model="gemini-3-pro-image-preview",
        prompt="Make the sky more dramatic with stormy clouds",
        image=image_input,
    )
    save("edit_image_gemini.png", edited_bytes)


async def test_stream_edit_image(client: MultiAIClient, base_image: bytes) -> None:
    """Test stream_edit_image, saving each progressive frame."""
    logger.info("=== stream_edit_image ===")

    image_input = ("base.png", base_image, "image/png")
    frame_index = 0
    async for frame_bytes in client.stream_edit_image(
        model="gpt-image-1",
        prompt="Transform the scene into a snowy winter landscape",
        image=image_input,
        quality="medium",
        output_format="png",
        partial_images=2,
    ):
        save(f"stream_edit_image_frame_{frame_index:02d}.png", frame_bytes)
        frame_index += 1

    logger.info(f"  → {frame_index} frame(s) received")


# ---------------------------------------------------------------------------
# Entry point
# ---------------------------------------------------------------------------

async def main() -> None:
    """Run all image tests sequentially."""
    client = build_client()

    # Generate a base image used by the edit tests
    base_image = await test_text_to_image(client)

    await test_stream_text_to_image(client)
    await test_edit_image(client, base_image)
    await test_edit_image_multi(client, base_image)
    await test_stream_edit_image(client, base_image)

    # Gemini tests — comment out if no Gemini key is available
    await test_text_to_image_gemini(client)
    await test_edit_image_gemini(client, base_image)

    logger.info(f"Total cost: ${client.total_usage:.4f}")
    logger.info(f"All images saved to ./{OUTPUT_DIR}/")


if __name__ == "__main__":
    asyncio.run(main())
